#include <stdio.h>
#include "socket.h"

#define LIN_ALLOW	1
#define LIN_DENY	2

#define LIN_TCP		1
#define LIN_UDP		2

#define LIN_WAIT	1
#define LIN_NOWAIT	2

#define	CONFIG_FILE	"lin.conf"

struct QGroup {
	char ip[256];
	QGroup *next;

	QGroup() { next=(QGroup*)NULL; }
};

struct QAccess {
	int access;
	char ipFile[256];
	QGroup *group;

	QAccess *next;
	QAccess() { group=(QGroup*)NULL; next=(QAccess*)NULL; }
};

struct QService {
	char ipFile[256];
	int uid;
	char service[30];
	char fields[4][256];

	QGroup *group;
	QService *next;

	QService() { group=(QGroup*)NULL; next=(QService*)NULL; }
};

struct QConfig {
	int port;
	int protocol;
	int serverType;
	int backlog;
	char accessFile[256];
	char serviceFile[256];

	QAccess* access;
	QService* service;
	QConfig *next;

	QConfig() { access=(QAccess*)NULL; service=(QService*)NULL; next=(QConfig*)NULL; }
};

class QConfigFile {
	QConfig *config;

	// To read data from Configuration Files
	bool readGroup(char *filename,QGroup **top);
	bool readAccess(char *filename,QAccess **top);
	bool readService(char *filename,QService **top);

	// To read data from Socket
	bool readGroupServer(QSocket &client,QGroup **top,char *name);
	bool readAccessServer(QSocket &client,QAccess **top,int port,char *protocol);
	bool readServiceServer(QSocket &client,QService **top,int port,char* protocol);

	// To parse entries and place them in linked-list
	bool parseAccessEntry(QAccess **top,char *buffer,QSocket *client=(QSocket*)NULL);
	bool parseServiceEntry(QService **top,char *buffer,QSocket *client=(QSocket*)NULL);
	bool parseConfigEntry(char *buffer,QSocket *client=(QSocket*)NULL);

	// To dispose off linked list entries - TOP function dispose() should be called
	// which will use these functions.
	void disposeGroup(QGroup**);
	void disposeAccess(QAccess**);
	void disposeService(QService**);
public:
	bool isMatch(char *pattern,char *ip);
	QConfig *current;
	QAccess *currentAccess;
	QService *currentService;
public:
	QConfigFile() { config=(QConfig*)NULL; }
	bool read(char *filename);
	void display();
	bool readServer(QSocket &client);

	bool get(int port,int protocol=LIN_TCP);
	bool rollback();
	bool next();
	bool rollbackAccess();
	bool nextAccess();
	bool rollbackService();
	bool nextService();

	bool isAllowed(int port,char *host);
	bool getServiceByConnection(int port,char *host);

	void dispose();
};

extern QConfigFile config;
