#include "socket.h"

//---------------------------------------------------------------------
QSocketException::QSocketException(int h) {
	handle=h;
}

//---------------------------------------------------------------------
QSocket::QSocket() {
	handle=-1; exception=false;
}
//---------------------------------------------------------------------
void QSocket::detach() {
	handle=-1;
	sourcePort=targetPort=targetHost[0]=0;
}

//---------------------------------------------------------------------
void QSocket::attach(int h) {
	close();

	sockaddr_in addr;
	unsigned int length=sizeof(addr);
	getpeername(h,(sockaddr*)&addr,&length);

	handle=h;
	targetPort=ntohs(addr.sin_port);
	sprintf(targetHost,"%d.%d.%d.%d",
		addr.sin_addr.s_addr&0xFF,
		(addr.sin_addr.s_addr>>8)&0xFF,
		(addr.sin_addr.s_addr>>16)&0xFF,
		(addr.sin_addr.s_addr>>24)&0xFF);

}
		
//---------------------------------------------------------------------
bool QSocket::create(int port,int backlog) {
	close();

	handle=socket(PF_INET,SOCK_STREAM,0);
	if (handle==-1) return false;
	sockaddr_in addr;
	addr.sin_family=AF_INET;
	addr.sin_port=htons(port);
	addr.sin_addr.s_addr=INADDR_ANY;
	if (bind(handle,(sockaddr*)&addr,sizeof(addr))==-1) {
		::close(handle); return false;
	}
	if (listen(handle,backlog)==-1) {
		::close(handle); return false;
	}
	sourcePort=port;
	return true;
}

//---------------------------------------------------------------------
bool QSocket::connectSOCKS(char *hostname,int port,char *sockshost,int socksport) {
	close();

	hostent *phost=gethostbyname(sockshost);
	if (!phost) {
		phost=gethostbyaddr(sockshost,strlen(sockshost),AF_INET);
		if (!phost) return false;
	}

	handle=socket(PF_INET,SOCK_STREAM,0);
	if (handle==-1) return false;

	sockaddr_in addr;
	addr.sin_family=AF_INET;
	addr.sin_port=htons(socksport);
	bcopy(phost->h_addr,&addr.sin_addr.s_addr,phost->h_length);
	if (::connect(handle,(sockaddr*)&addr,sizeof(addr))==-1) {
		::close(handle); return false;
	}

	char temp[256];
	short int p;
	FILE *file=fdopen(handle,"rb+");
	setvbuf(file,NULL,_IONBF,0);

	try {
		temp[0]=5; temp[1]=1; temp[2]=0;
		fwrite(temp,3,1,file); fflush(file);

		fread(temp,2,1,file); fflush(file);
		if (temp[0]!=5 || temp[1]!=0) throw QSocketException();

		temp[0]=5; temp[1]=1; temp[2]=0; temp[3]=3;
		temp[4]=strlen(hostname); strcpy(&temp[5],hostname);
		*((unsigned short int*)&temp[5+strlen(hostname)])=htons(port);
		fwrite(temp,strlen(hostname)+7,1,file); fflush(file);

		fread(temp,2,1,file); fflush(stdout);
		if (temp[0]!=5 || temp[1]!=0) {
			printf("Reply: %d\n",temp[1]);
			throw QSocketException();
		}
		fread(&temp[0],1,1,file);
		fread(&temp[0],1,1,file);
		switch(temp[0]) {
		case 1: fread(temp,4,1,file); break;
		case 3: fread(&temp[0],1,1,file); read(temp,(int)temp[0]); break;
		case 4: fread(temp,16,1,file); break;
		default: throw QSocketException();
		}
		return true;
	}
	catch(...) {
		fclose(file);
		close();
		return false;
	}
	fclose(file);
	return true;
}

//---------------------------------------------------------------------
bool QSocket::connect(char *hostname,int port) {
	hostent *phost;

	close();

	phost=gethostbyname(hostname);
	if (!phost) {
		phost=gethostbyaddr(hostname,strlen(hostname),AF_INET);
		if (!phost) return false;
	}

	handle=socket(PF_INET,SOCK_STREAM,0);
	if (handle==-1) return false;

	sockaddr_in addr;
	addr.sin_family=AF_INET;
	addr.sin_port=htons(port);
	bcopy(phost->h_addr,&addr.sin_addr.s_addr,phost->h_length);
	if (::connect(handle,(sockaddr*)&addr,sizeof(addr))==-1) {
		::close(handle); return false;
	}
	strcpy(targetHost,hostname);
	targetPort=port;
	return true;
}

//---------------------------------------------------------------------
int QSocket::accept() {
	sockaddr_in addr;
	unsigned int length=sizeof(addr);

	int res=::accept(handle,(sockaddr*)&addr,&length);
	if (res!=-1) {
		targetPort=ntohs(addr.sin_port);
		sprintf(targetHost,"%d.%d.%d.%d",
			addr.sin_addr.s_addr&0xFF,
			(addr.sin_addr.s_addr>>8)&0xFF,
			(addr.sin_addr.s_addr>>16)&0xFF,
			(addr.sin_addr.s_addr>>24)&0xFF);
	}
	return res;
}

//---------------------------------------------------------------------
void QSocket::close() {
	if (handle!=-1) {
		shutdown(handle,2);
		targetHost[0]=0;
		sourcePort=targetPort=0;
	}
	handle=-1;
}

//---------------------------------------------------------------------
int QSocket::read(void *buffer,int length) {
	int res=recv(handle,buffer,length,0);
	if (res<=0) {
		if (exception) throw QSocketException(handle);
		return -1;
	}
	return res;
}

//---------------------------------------------------------------------
int QSocket::write(void *buffer,int length) {
	if (length==-1) length=strlen((char*)buffer);
	int res=send(handle,buffer,length,0);
	if (res<=0) {
		if (exception) throw QSocketException(handle);
		return -1;
	}
	return res;
}


//---------------------------------------------------------------------
// socks should be blocking for this function call to work properly
int QSocket::readline(char *buffer,int length) {
	if (length<=0) return -1;

	int count=0,res;
	while(1) {
		if (count==length) break;
		res=recv(handle,&buffer[count],1,0);
		if (res<=0) {
			if (exception) throw QSocketException(handle);
			return -1;
		}
		if (buffer[count]=='\n') break;
		count++;
	}
	buffer[count]=0;
	for(int i=0;i<strlen(buffer);i++)
		if (buffer[i]=='\n' || buffer[i]=='\r')
			buffer[i]=0;
	return 1;
}

//---------------------------------------------------------------------
int QSocket::connectService(int handle1,int handle2) {
	fd_set fd,ready;
	int maxfd,res,total=0;
	char buffer[100];

	FD_ZERO(&fd);
	FD_SET(handle1,&fd);
	FD_SET(handle2,&fd);
	maxfd=handle1<handle2 ? handle2 : handle1;

	while(1) {
		bcopy(&fd,&ready,sizeof(fd));
		select(maxfd+1,&ready,(fd_set*)NULL,(fd_set*)NULL,(timeval*)NULL);

		if (FD_ISSET(handle1,&ready)) {
			res=recv(handle1,buffer,100,0);
			if (res<=0) break;
			total+=res;
			res=send(handle2,buffer,res,0);
			if (res<=0) break;
		}
		if (FD_ISSET(handle2,&ready)) {
			res=recv(handle2,buffer,100,0);
			if (res<=0) break;
			total+=res;
			res=send(handle1,buffer,res,0);
			if (res<=0) break;
		}
	}
	shutdown(handle1,2);
	shutdown(handle2,2);
	return total;
}

//---------------------------------------------------------------------
int QSocket::connectService(QSocket &s1,QSocket& s2) {
	return connectService(s1.handle,s2.handle);
}


